import {Material} from './material.model'
import {LocalDate} from 'js-joda'
import {Company} from './company.model'
import {EmployeeGroup} from './employee'

export class Recipe {
  constructor(
    public id: number,
    public name: string,
    public description: string,
    public color: string,
    public gloss: number,
    public sample: number,
    public approbationDate: LocalDate,
    public remark: string,
    public company: Company,
    public mixes: Mix[],
    public groupsInformation: RecipeGroupInformation[]
  ) {
  }
}

export class RecipeGroupInformation {
  constructor(
    public id: number,
    public group: EmployeeGroup,
    public note: string,
    public steps: RecipeStep[]
  ) {
  }
}

export class Mix {
  constructor(
    public id: number,
    public mixType: MixType,
    public mixMaterials: MixMaterial[],
    public location: string,
  ) {
  }
}

export class MixMaterial {
  constructor(
    public id: number,
    public material: Material,
    public quantity: number,
    public position: number
  ) {
  }
}

export class MixMaterialDto {
  constructor(
    public materialId: number,
    public quantity: number,
    public isPercents: boolean,
    public position: number
  ) {
  }
}

class MixType {
  constructor(
    public id: number,
    public name: string,
    public material: Material
  ) {
  }
}

export class RecipeStep {
  constructor(
    public id: number,
    public message: string,
    public position: number
  ) {
  }
}

export function recipeMixCount(recipe: Recipe): number {
  return recipe.mixes.length
}

export function recipeStepCount(recipe: Recipe): number {
  return recipe.groupsInformation.length > 0 ? recipe.groupsInformation.flatMap(i => i.steps).length : 0
}

export function recipeNoteForGroupId(recipe: Recipe, groupId: number): string | null {
  const groupInformation = recipe.groupsInformation.find(i => i.group.id === groupId)
  return groupInformation ? groupInformation.note : null
}

export function recipeStepsForGroupId(recipe: Recipe, groupId: number): RecipeStep[] {
  const groupInformation = recipe.groupsInformation.find(i => i.group.id === groupId)
  return groupInformation ? sortRecipeSteps(groupInformation.steps) : []
}

export function sortRecipeSteps(steps: RecipeStep[]): RecipeStep[] {
  return steps.sort((a, b) => a.position - b.position)
}

export function mixMaterialsAsMixMaterialsDto(mix: Mix): MixMaterialDto[] {
  return sortMixMaterialsDto(mix.mixMaterials.map(m => new MixMaterialDto(
    m.material.id,
    m.quantity,
    m.material.materialType.usePercentages,
    m.position
  )))
}

export function sortMixMaterialsDto(mixMaterials: MixMaterialDto[]): MixMaterialDto[] {
  return mixMaterials.sort((a, b) => a.position - b.position)
}

